/**********************************************************************
 *
 * ResultCanvas.java -- a Canvas onto which a phrase list is drawn
 *
 * Copyright 1997-2000 Gordon W. Paynter
 * Copyright 2000 The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

/*********************************************************************

This class is used in the Phind java applet (Phind.java).

The results of a query are displayed on a ResultCanvas object.  Each 
line of the result is stored in a ResultItem, and the ResultCanvas 
contains a Vector of ResultItems.

Each ResultCanvas is embedded in a ResultBox alongside a Scrollbar.  
When the ResultCanvas is drawn, it looks at the scrollbar, calculates 
which ResultItems are visible, then draws them on the screen.

**********************************************************************/


import java.awt.Canvas;
import java.awt.Scrollbar;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Dimension;
import java.awt.Color;
import java.util.Vector;
import java.awt.Image;
import java.awt.Event;

import java.net.URL;
import java.applet.Applet;
import java.util.Date;

public class ResultCanvas extends Canvas {

    // Other objects associated with this 
    Phind phind;
    ResultBox parent;
    Scrollbar scrollbar;
    
    // fonts and font spacings to use
    Font areaPlain, areaBold;
    int lineSpacing;
    
    // The phrases to display on this canvas
    int numberOfPhrases;
    int firstPhraseDisplayed;
    int phraseSelected;
    Vector phrases;
    
    // the background image
    public static Image backgroundImage;
    
    
    // Create a ResultCanvas from the ResultBox which is its parent.
    ResultCanvas(ResultBox p) {
	
	parent = p;
	phind = p.phind;
	scrollbar = p.s;
	parent.disableScrollbar();

	areaPlain = phind.plainFont;
	areaBold  = phind.boldFont;

	lineSpacing = phind.fontSize + 2;

	phrases = new Vector();
	numberOfPhrases = 0;
	firstPhraseDisplayed = 0;
	phraseSelected = -1;
    
	if (backgroundImage == null) {
	    backgroundImage = p.phind.backgroundImage;
	}

    }

    void resetCanvas( ) {
	phrases.removeAllElements();
	numberOfPhrases = 0;
	repaint();
    }


    // add a new item of input.
    // return true if successful, otherwise false.
    boolean addResultItem( ResultItem item ) {
 
	// Add a new result, in sorted order.
	// First sort key is item kind (lowest first),
	// second is frequency (highest first).
	// This is not efficient, but I don't care right now.
	int index = 0;
	while ((index < numberOfPhrases) &&
	       ((item.kind > ((ResultItem) phrases.elementAt(index)).kind) ||
		((item.kind == ((ResultItem) phrases.elementAt(index)).kind) &&
		 (item.freq <= ((ResultItem) phrases.elementAt(index)).freq)))) {
	    index++;
	}
	phrases.insertElementAt(item,index);
	numberOfPhrases++;
	return true;
    }


    // Make sure the more Phrases item only appears in the list
    // if it is meant to.
    void updateMorePhrasesMarker() {
    
	// System.out.println("updateMorePhrasesMarker() ");

	// look for a marker
	boolean found = false;
	int index = 0;
	while (!found && (index < numberOfPhrases)) {
	    if (((ResultItem) phrases.elementAt(index)).isMorePhrases()) {
		// System.out.println("found marker at: " + index + " of " + numberOfPhrases);
		found = true;
	    } else {
		index++;
	    }
	}

	if (parent.expansionsRetrieved == parent.numberOfExpansions) {
	    // there should be no marker
	    // System.out.println("No marker needed");
	    if (found) {
		// System.out.println("Remove marker");
		phrases.removeElementAt(index);
		numberOfPhrases--;
	    }

	} else if (parent.expansionsRetrieved < parent.numberOfExpansions) {
	    // there should be a marker
	    // System.out.println("Needs marker");

	    if (!found) {
		// System.out.println("Add marker");
		ResultItem ri = new ResultItem(ResultItem.morePhrases, parent.searchKey);
		addResultItem(ri);
	    }
	}
    }


    // Make sure the more documents marker appears when required
    void updateMoreDocumentsMarker() {
    
	// System.out.println("updateMoreDocumentsMarker() ");

	// look for a marker
	boolean found = false;
	int index = 0;
	while (!found && (index < numberOfPhrases)) {
	    if (((ResultItem) phrases.elementAt(index)).isMoreDocuments()) {
		found = true;
	    } else {
		index++;
	    }
	}

	if (parent.documentsRetrieved == parent.numberOfDocuments) {
	    // there should be no marker
	    if (found) {
		phrases.removeElementAt(index);
		numberOfPhrases--;
	    }

	} else if (parent.documentsRetrieved < parent.numberOfDocuments) {
	    // there should be a marker
	    if (!found) {
		ResultItem ri = new ResultItem(ResultItem.moreDocuments, parent.searchKey);
		addResultItem(ri);
	    }
	}
    }

      

 


    public void update(Graphics g) {
	paint(g);
    }

    public void paint(Graphics g) {

	// calculate the canvas size, margins, and spacing
	
	// Note: the size() method is deprecated, but getSize fails in NetScape 4
	// Dimension canvasSize = getSize();
	Dimension canvasSize = size();

	g.setFont(areaPlain);
	int space = g.getFontMetrics().stringWidth(" ");
	int margin = g.getFontMetrics().stringWidth(" 8888 ");

	int leftMargin = 0;
	int secondColumn = canvasSize.width - margin;
	int firstColumn = secondColumn - margin;
	int rightMargin = firstColumn;

	// calculate the number of values that will fit onscreen
	int visible = canvasSize.height / lineSpacing;

	// get the initial scrollbar setting
	// and calculate the first result to output
	int scrollValue = scrollbar.getValue();
	if (numberOfPhrases <= visible) {
	    scrollValue = 0;
	} else if (scrollValue > (numberOfPhrases - visible)) {
	    scrollValue = numberOfPhrases - visible;
	} 
	firstPhraseDisplayed = scrollValue;

	// Draw the phrase area

	// draw the background
	if (phind.showImage) {
	    try {
		g.drawImage(backgroundImage, 
			    leftMargin, 0, rightMargin, canvasSize.height, 
			    Color.white, null);
	    } catch (Exception e) {
		phind.showImage = false;
		System.err.println("ResultCanvas paint error: " + e); 
		g.setColor(Color.white);
		g.fillRect(0, 0, canvasSize.width, canvasSize.height);
	    }
	} else {
	    g.setColor(Color.white);
	    g.fillRect(0, 0, canvasSize.width, canvasSize.height);
	}
    
	// If there are no phrases, output a brief explanation.
	if (numberOfPhrases == 0) {
	    g.drawString("No phrases match this query.", leftMargin + 10, lineSpacing);
	}

	// Output each of the visible ResultItems
	ResultItem result;
	int tab, i, y = 0;
	int center = leftMargin 
	    + ((rightMargin - leftMargin 
		- g.getFontMetrics().stringWidth(parent.searchPhrase)) / 2);
	
	for (i = scrollValue; 
	     (i < numberOfPhrases) && (y + lineSpacing < canvasSize.height); i++) {

	    // Get the resultItem to output
	    result = (ResultItem) phrases.elementAt(i);
	    
	    // Graphics settings for drawing this line
	    y += lineSpacing;
	    g.setFont(areaPlain);

	    // Highlight if the selected phrase.
	    if (i == phraseSelected) {
		g.setColor(new Color(255, 220, 115));
		g.fillRect(leftMargin, y-lineSpacing+2, rightMargin, lineSpacing);
		g.setColor(Color.black);
	    }

	    // Draw the item

	    if (result.isPhrase()) {
		// The item is a Phrase
		tab = center - g.getFontMetrics().stringWidth(result.prefix) - space;
		g.drawString(result.prefix, tab, y);
		g.setFont(areaBold);
		g.drawString(result.body, center, y);
		tab = center + space + g.getFontMetrics().stringWidth(result.body);
		g.setFont(areaPlain);
		g.drawString(result.suffix, tab, y);
	
	    } else if (result.isDocument()){
		// The item is a URL
		g.setFont(areaPlain);
		g.setColor(Color.blue);
		tab = (rightMargin - g.getFontMetrics().stringWidth(result.body)) / 2;
		g.drawString(result.body, tab, y);
		g.setColor(Color.black);
      
	    } else if (result.isMorePhrases()){
		// The item is a More documents marker
		g.setColor(new Color(255, 200, 200));
		g.fillRect(leftMargin, y-lineSpacing+2, rightMargin, lineSpacing);
		g.setColor(Color.black);

		g.setFont(areaPlain);
		tab = (rightMargin - g.getFontMetrics().stringWidth("Get more phrases")) / 2;
		g.drawString("Get more phrases", tab, y);

	    } else if (result.isMoreDocuments()){
		// The item is a More documents marker
		g.setColor(new Color(150, 193, 156));
		g.fillRect(leftMargin, y-lineSpacing+2, rightMargin, lineSpacing);
		g.setColor(Color.black);

		g.setFont(areaPlain);
		tab = (rightMargin - g.getFontMetrics().stringWidth("Get more documents")) / 2;
		g.drawString("Get more documents", tab, y);

	    }

	}

	// Draw the frequecy columns

	// column backgrounds
	g.setColor(new Color(235, 245, 235));
	g.fillRect(firstColumn, 0, margin, canvasSize.height);
	g.setColor(new Color(200, 220, 200));
	g.fillRect(secondColumn, 0, margin, canvasSize.height);

	// fill in the numbers
	g.setColor(Color.black);
	g.setFont(areaPlain);
	y = 0;

	for (i = scrollValue; 
	     (i < numberOfPhrases) && (y + lineSpacing < canvasSize.height); i++) {
	    
	    // Get the resultItem to output
	    result = (ResultItem) phrases.elementAt(i);
	    
	    // Graphics settings for drawing this line
	    y += lineSpacing;

	    // Write the document frequency
	    if ((result.documents > 1) || result.isPhrase()) {
		g.drawString(" " + Integer.toString(result.documents), firstColumn, y);
	    }

	    // Write the term frequency
	    if (result.freq > 0) {
		g.drawString(" " + Integer.toString(result.freq), secondColumn, y);
	    }
	}

	// Adjust the scrollbar
	if (visible >= numberOfPhrases) {
	    parent.disableScrollbar();
	} else {
	    scrollbar.setValues(scrollValue, visible, 0, numberOfPhrases);
	    // Deprecated - setPageIncrement() has been replaced by
	    // setBlockIncrement(), but the latter doesn't work on some
	    // browsers.  Damn.
	    // scrollbar.setBlockIncrement(visible - 1);
	    scrollbar.setPageIncrement(visible - 1);
	    // Deprecated - more nonsense: enable() has been replaced by
	    // setEnabled(true), but this doesn't work on some browsers.
	    //scrollbar.setEnabled(true);
	    scrollbar.enable();
	}

	g.drawRect(0,0, canvasSize.width - 1, canvasSize.height - 1);
    }


    // User interaction
    //
    // All interaction with the ResultCanvas is therough mouse clicks, and
    // is handles in this method.  Note we ignore clicks that follow
    // another too closely to avoid problems with slow connections.
    public boolean handleEvent(Event event) {
    
	if (event.id == Event.MOUSE_UP) {

	    // ignore actions that occur within 1 second of the last
	    Date now = new Date();
	    // System.out.println("Click time: " + now.toString());
	    if (now.getTime() < (phind.lastQueryEndTime.getTime() + 1000)) {
		System.out.println("Ignoring click - too close to last query.");
		return true;
	    }

	    // which Item is selected?
	    int rowSelected = event.y / lineSpacing;
	    int itemSelected = rowSelected + firstPhraseDisplayed;
	    ResultItem item = (ResultItem) phrases.elementAt(itemSelected);

	    // System.out.println("Select: " + String.valueOf(rowSelected) );
	    if (itemSelected <= numberOfPhrases) {
	
		// What do we do with the event?
		// If user clicks on a phrase, we expand the phrase
		// or send the phrase to a search engine (right click)

		if (item.isPhrase()) {
		    // Expand the phrase

		    phraseSelected = itemSelected;
		    update(getGraphics());
	  
		    parent.lookupPhrase(item.rule, item.toString(), 2);

		    if (event.metaDown()) { 
			// Also send to search engine
			if (phind.search_url.equals("none") || phind.search_url.equals("")) {
			    System.out.println("No searching in this collection");
			} else {
			    URL url;
			    try { 
				url = new URL(phind.search_url + "%22" + item.toString() + "%22&");
				phind.getAppletContext().showDocument(url, "phindsearch");
				System.out.println("The query is: " + url.toString());
			    } catch (Exception e) { 
				System.out.println("URL error: " + e.toString());
			    }
			} 
		    }

		} else if (item.isDocument()) {
		    // The user clicks on a URL; display it.
		    phraseSelected = itemSelected;
		    update(getGraphics());

		    URL url;
		    try {
			String address = phind.library_address 
			    + "a=d&c=" + phind.collection 
			    + "&d=" + item.rule
			    + "&q=" + parent.searchPhrase.replace(' ', '+');
			url = new URL(address);
			System.out.println("URL selected: " + url.toString());
			phind.getAppletContext().showDocument(url);
		    } catch (Exception e) {
			System.out.println("URL error: " + e.toString());
		    }

		} else if (item.isMorePhrases()){
		    // The user clicks on a "get more phrases" marker.
		    // We have to send a new query to the host
		    parent.lookupPhrase(parent.searchKey, parent.searchPhrase, 3);

		} else if (item.isMoreDocuments()){
		    parent.lookupPhrase(parent.searchKey, parent.searchPhrase, 4);


		}
		repaint();
	    }
	}
	return true;
    }    
  
}



